package swingmovinggrapheditor.graph.moving;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.TimerTask;
import swingmovinggrapheditor.graph.Arc;
import swingmovinggrapheditor.graph.Node;

public abstract class MovingCharacter implements Serializable {

    private char character;
    private Node<MovingCharacter> node;
    public static List<CharacterMover> movers = new ArrayList<>();

    public MovingCharacter(char character, Node<MovingCharacter> node) {
        this.character = character;
        setNode(node);
    }

    public MovingCharacter(char character) {
        this(character, null);
    }

    public final void setNode(Node<MovingCharacter> node) {
        this.node = node;
        if (node != null) {
            CharacterMover characterMover = new CharacterMover();
            new Thread(characterMover).start();
            movers.add(characterMover);
        }
    }

    @Override
    public String toString() {
        return String.valueOf(character);
    }

    public void setCharacter(char character) {
        this.character = character;
    }

    public char getCharacter() {
        return character;
    }

    public class CharacterMover implements Runnable {

        private boolean running = true;

        public void stop() {
            running = false;
        }

        @Override
        public void run() {
            while (running) {
                move();
                try {
                    Thread.sleep(40);
                } catch (InterruptedException ex) {
                    break;
                }
            }
        }
    }

    private class CharMoverTask extends TimerTask {

        @Override
        public void run() {
            move();
        }
    }

    private static Random rnd = new Random();
    private void move() {
        if (character == ' ') {
            try {
                Thread.sleep(200);
            } catch (InterruptedException ex) {
                return;
            }
            return;
        }
        if (node == null) {
            return;
        }
        if (node.arcs.isEmpty()) {
            return;
        }
        Arc<MovingCharacter, ? extends Node<MovingCharacter>> arcToMoveOn = node.arcs.get(rnd.nextInt(node.arcs.size()));
        if (arcToMoveOn.edge1 == arcToMoveOn.edge2) {
            return;
        }
        Node<MovingCharacter> newNode;
        if (arcToMoveOn.edge1 == node) {
            newNode = arcToMoveOn.edge2;
        } else {
            newNode = arcToMoveOn.edge1;
        }
        if (newNode.element.character != ' ') {
            return;
        }
        newNode.element.character = character;
        character = ' ';
        nodeMoved();
    }

    protected abstract void nodeMoved();
}
