package swingmovinggrapheditor.gui;

import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Collections;
import javax.swing.JFrame;
import javax.swing.JPanel;
import swingmovinggrapheditor.graph.LocatedNode;
import swingmovinggrapheditor.graph.moving.MovingCharacter;

/**
 *
 * @author trust56
 */
public class GraphEditorFrame extends JFrame {

    SwingGraph<MovingCharacter> graph;

    public GraphEditorFrame() {
        setTitle("Gráf szerkesztő");
        setLocation(40, 40);
        setSize(600, 400);
        addWindowListener(windowAdapter);
        setContentPane(graphPanel);
        graphPanel.setDoubleBuffered(true);
        graphPanel.addMouseListener(graphPanelMouseAdapter);
        graphPanel.addMouseMotionListener(graphPanelMouseAdapter);
        graphPanel.setFocusable(true);
        graphPanel.addKeyListener(graphPanelKeyListener);
        graph = new SwingGraph<MovingCharacter>() {
            @Override
            public void graphChanged() {
                graphPanel.repaint();
            }
        };
    }
    private WindowListener windowAdapter = new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
            //MovingCharacter.getTimer().cancel();
            for (MovingCharacter.CharacterMover characterMover : MovingCharacter.movers) {
                characterMover.stop();
            }
            GraphEditorFrame.this.dispose();
        }
    };
    private JPanel graphPanel = new JPanel() {
        @Override
        public void paint(Graphics g) {
            //super.paint(g);
            g.setColor(Color.LIGHT_GRAY);
            g.fillRect(0, 0, getWidth(), getHeight());
            if (mouseDraggedPoint != null) {
                g.setColor(Color.BLUE);
                int x1 = mousePressedPoint.x;
                int y1 = mousePressedPoint.y;
                int x2 = mouseDraggedPoint.x;
                int y2 = mouseDraggedPoint.y;
                g.drawLine(x1, y1, x2, y2);
            }
            graph.paint(g);
        }
    };
    private Point mousePressedPoint = new Point(0, 0);
    private Point mouseDraggedPoint = null;
    private MouseAdapter graphPanelMouseAdapter = new MouseAdapter() {
        private final int DISTANCE_TRESHOLD = 10;
        private boolean mouseDragged = false;
        private int mousePressedButton;

        @Override
        public void mouseClicked(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON1) {
                int x = e.getX();
                int y = e.getY();
                MovingCharacter movingCharacter = new SwingMovingCharacter(' ', null);
                LocatedNode<MovingCharacter> newNode = graph.addNode(x, y, movingCharacter);
                if (newNode != null) {
                    movingCharacter.setNode(newNode);
                }
            } else if (e.getButton() == MouseEvent.BUTTON3) {
                graph.addArc(Utils.p2p(e.getPoint()));
            }
        }

        @Override
        public void mousePressed(MouseEvent e) {
            mouseDragged = false;
            mousePressedPoint = e.getPoint();
            mousePressedButton = e.getButton();
        }

        @Override
        public void mouseMoved(MouseEvent e) {
            graph.selectPoint(Utils.p2p(e.getPoint()));
        }

        @Override
        public void mouseDragged(MouseEvent e) {
            mouseDragged = !SwingGraph.pointsCloserThan(DISTANCE_TRESHOLD,
                    Utils.p2p(e.getPoint()), Utils.p2p(mousePressedPoint));
            if (mousePressedButton == MouseEvent.BUTTON1) {
                graph.dragPoint(Utils.p2p(e.getPoint()));
            } else if (mousePressedButton == MouseEvent.BUTTON3) {
                mouseDraggedPoint = e.getPoint();
                graphPanel.repaint();
            }
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            if (mouseDragged) {
                if (e.getButton() == MouseEvent.BUTTON3) {
                    graph.addArc(Utils.p2p(mouseDraggedPoint));
                }
            } else {
                mouseClicked(e);
            }
            mouseDraggedPoint = null;
            graphPanel.repaint();
        }
    };
    private KeyListener graphPanelKeyListener = new KeyAdapter() {
        @Override
        public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_DELETE) {
                graph.deleteSelectedNodeOrArc();
            } else {
                if (graph.getSelectedNode() != null) {
                    graph.getSelectedNode().element.setCharacter(e.getKeyChar());
                    repaintGraphPanel();
                }
            }
        }
    };

    void repaintGraphPanel() {
        graphPanel.repaint();
    }

    private class SwingMovingCharacter extends MovingCharacter {

        public SwingMovingCharacter(char keyChar, LocatedNode<MovingCharacter> selectedNode) {
            super(keyChar, selectedNode);
        }

        @Override
        protected void nodeMoved() {
            EventQueue.invokeLater(new Runnable() {
                @Override
                public void run() {
                    graphPanel.repaint();
                }
            });
            ArrayList<Character> chars = new ArrayList<>();
            for (LocatedNode<MovingCharacter> node : graph.getNodes()) {
                chars.add(node.element.getCharacter());
            }
            Collections.sort(chars);
            System.out.println(chars);
        }
    }
}
