/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package javaapplication1;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * Alapprobléma: Adott egy Gyártó(Producer) és Fogyasztó(Consumer) akiket
 * egy gyártószalag választ el egymástól. A Gyártó feladata, hogy termékeket 
 * helyezzen el a szalagon, ha az még nincs tele. A Fogyasztó feladat, hogy a
 * már gyártószalagra helyezett termékeket levegye, persze csak akkor, ha a 
 * szalag nem üres. Figyeljük meg, hogy a nem üres, és nem telitett állapotok
 * pont kizárják egymást!
 * @author puzsaai
 */
public class ProducerConsumer {
    
    /**
     * Gyártó osztály
     * összesen maxProducts terméket akar elhelyezni a szalagon.
     */
     private static class Producer extends Thread{
        private static Random rand = new Random();
        private MyList list;
        private static final int maxProducts = 20;
        
        public Producer(MyList list){
            this.list = list;//gyártósor
        }
        
        @Override
        public void run(){
            for(int j = 0; j < maxProducts; j++){
                int i = rand.nextInt(100);
                System.out.println(Thread.currentThread().getName() + " belerak: " + i );
                list.put(i);//helyezzük a random számot a szalagra...ha ez lehetséges
            }
            list.put(-1);//vége a termelésnek
        }
    }
    
     
     /**
      * Fogyasztó osztály
      * Amíg nincs vége a termelésnek (-1et nem vesz le a gyártósorról),
      * Addig próbálkozik terméket levenni a gyártósorról!
      */
    private static class Consumer extends Thread{
        private MyList list;
        
        public Consumer(MyList list){
            this.list = list;//gyártósor
        }
        
        @Override
        public void run(){
            int last = list.get();//vegyük le az első terméket
            while(last != -1){//amíg nincs vége a termelésnek
                System.out.println(Thread.currentThread().getName() + " kivette:" + last);
                last = list.get();//vegyük le a következőt...ha ez lehetséges
            }
        }
    }
    
    /**
     * Gyártósor osztály
     */
    private static class MyList{
        public static final int LENGTH = 10;//maximum ennyi termék lehet egyszerre rajta
        List<Integer> list;
        
        public MyList(){
            list = new ArrayList(LENGTH);
        }
        
        //a put() és get() metódusok synchronized-ak. Tehát összesen csak 1 szál
        //futhat bennük! Ha egy másik is be akar lépni valamelyik blokkba, akkor várnia
        //kell, amíg a már benne lévő ki nem lép. (monitor)
        
        /**
         * Ezt hívja meg a Termelő.
         * Ha LENGTH termék van a szalagon, akkor blokkolódik, mert nem tudja felrakni 
         * a terméket a szalagra. Ekkor majd a fogyasztó fogja felkelteni, ha 
         * levett egy terméket a szalagról (az már nincs tele).
         * @param element - termék
         */
        public synchronized void put(int element){
            while(list.size() == LENGTH){//Televan a szalag
                try {
                    System.out.println(Thread.currentThread().getName() + " várakozik");
                    wait();//várunk, amíg fel nem ébreszt a fogyasztó
                } catch (InterruptedException ex) {}
            }
            
           System.out.println(Thread.currentThread().getName() + " belerakta: " + element);
           list.add(element);//rakjunk terméket a szalagra
           notify();//értesítsünk az esetleg váró fogyasztót, hogy már van termék
           //a szalagon, amit le tudnak venni!
        }
        
        
        /**
         * Ezt hívja meg a fogyasztó. Ha lehetséges akkor levesz egy terméket a
         * szalagról. Ha viszont nem, akkor vár amíg egy termelő nem szól, hogy 
         * terméket helyezett el a szalagon.
         * @return 
         */
        public synchronized int get() {
            while(list.isEmpty()){//üres a szalag?
                try {
                    System.out.println(Thread.currentThread().getName() + " várakozik");
                    wait();//ha igen, akkor várunk
                } catch (InterruptedException ex) {}
            }
            System.out.println(Thread.currentThread().getName() + " kivesz");
            int ret = list.remove(0);//levesszük az elsőt
            notify();//értesítjük az esetleg várakozó termelőket, hogy már biztosan
            //nincs tele a szalag
            return ret;
        }
    }
    
    public static void main(String[] args){
        MyList list = new MyList();//gyártószalag
        Producer p = new Producer(list);//termelő
        Consumer c = new Consumer(list);//fogyasztó
        c.start();p.start(); 
    }
}
