package javaapplication1;

import java.awt.EventQueue;
import java.awt.HeadlessException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.Scanner;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingWorker;

/*
 * Készítsünk programot, amely beolvassa a www.elte.hu weboldal html forrását,
 * és beírja egy grafikus ablak szövegdobozába
 */
public class SwingWorkerExample extends JFrame {

    public static void main(String[] args) {
        // A grafikus felületen bármilyen műveletet csak az eseménykezelést
        // vezérlő szálon szabad végezni. Ennek az alábbi a módja:
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                try {
                    new SwingWorkerExample().setVisible(true);
                } catch (HeadlessException | MalformedURLException ex) {
                    System.err.println(ex.getMessage());
                }
            }
        });
    }
    private final JTextArea textArea;
    private final URL url;

    public SwingWorkerExample() throws HeadlessException, MalformedURLException {
        url = new URL("http://www.elte.hu/");
        setTitle(url.toString());
        setSize(800, 600);
        setLocation(30, 30);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        textArea = new JTextArea();
        textArea.setLineWrap(true);
        getContentPane().add(new JScrollPane(textArea,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
        // A SwingWorker execute() függvénye meghívható az eseményvezérlésben,
        // mert az elindít egy párhuzamos szálat, amely a doInBackground()-ot
        // hajtja végre.
        new WebSiteReader().execute();
    }

    class WebSiteReader extends SwingWorker<Void, String> {

        /*
         * Ez a függvény definiálja, hogy milyen műveletet kell a háttérben,
         * az eseményvezérléstől függetlenül elvégezni.
         */
        @Override
        protected Void doInBackground() throws Exception {
            InputStream htmlStream = url.openStream();
            try (Scanner scanner = new Scanner(htmlStream)) {
                while (scanner.hasNextLine()) {
                    String htmlLine = scanner.nextLine();
                    // ha a részeredményeket a grafikus felületen szeretnénk
                    // látni, akkor a publish() függvénnyel téríthetjük vissza
                    // az eseményvezérlés szálába
                    publish(htmlLine); 
                }
            }
            return null;
        }

        /*
         * A publish() függvény által produkált részeredményeket ez a függvény
         * dolgozza fel. Mivel az eseményvezérlési szálon fut, csak megjelenítést
         * végezhet.
         */
        @Override
        protected void process(List<String> chunks) {
            for (String string : chunks) {
                textArea.append(string);
                textArea.append("\n");
            }
        }
    }
}